#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <stdarg.h>
#include <string.h>
#include <pthread.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
/* include local files */
#include "SFont.h"
#include "geometry.h"
#include "gfx_functions.h"

/***** global variables *****/
int main(int argc, char *argv[]){
	int returnval = 0, n, closest, bestdistance, distance, mx, my, cx, cy, cz;
	int numcorners = 3, x, y, middlex, last_button_state = 0, current_area = 0;
	point_class<int> *polypoints, *readpoints, *drawpoints;
	float ang = 0, sine, cosine, radius;
	int *depth_plane;

	SDL_Surface *test_image = NULL;

	/**** handle command line arguments ****/
	if(argc > 1){
		numcorners = atoi(argv[1]);
	}
	if(numcorners < 3)
		numcorners = 3;

	/**** initialize memory de-allocation ****/
	numfreepointers = 0;
	atexit(gfx_free_memory);

	/**** load our test image and allocate corners ****/
	test_image = gfx_load_png("BeachPenguin.png");
	if(test_image != NULL) free_pointer[numfreepointers++] = (void *)test_image;
	else returnval = 1;
	polypoints = (point_class<int> *)malloc(numcorners * sizeof(point_class<int>));
	if(polypoints) free_pointer[numfreepointers++] = (void *)polypoints;
	else returnval = 1;
	readpoints = (point_class<int> *)malloc(numcorners * sizeof(point_class<int>));
	if(readpoints) free_pointer[numfreepointers++] = (void *)readpoints;
	else returnval = 1;
	drawpoints = (point_class<int> *)malloc(numcorners * sizeof(point_class<int>));
	if(drawpoints) free_pointer[numfreepointers++] = (void *)drawpoints;
	else returnval = 1;

	/**** initialize graphics and events****/
	returnval |= gfx_init(800, 600, 32, 0);
	gfx_initialize_events();

	/*** allocate the Z buffer ***/
	if(!returnval){
		depth_plane = (int *)malloc(x_resolution * y_resolution * sizeof(int));
		if(depth_plane){
			free_pointer[numfreepointers++] = (void *)depth_plane;
			memset((void *)depth_plane, 1000, x_resolution * y_resolution * sizeof(int));
		}
		else returnval = 1;
	}

	/**** main routine ****/
	if(!returnval){
		middlex = test_image->w + 10;

		gfx_blit(screen_image, 0, 0, test_image);

		x = middlex + ((x_resolution - middlex) >> 1);
		y = y_resolution >> 1;
		radius = test_image->w;
		if(test_image->h < radius) radius = test_image->h;
		radius /= 2;
		for(n = 0; n < numcorners; n++){
			polypoints[n].x = (int)(x  + radius * sin(2 * pi * (float)n / (float)numcorners));
			polypoints[n].y = (int)(y  + radius * cos(2 * pi * (float)n / (float)numcorners));
			polypoints[n].z = 0;
		}

		x = test_image->w >> 1;
		y = test_image->h >> 1;
		radius = test_image->w;
		if(test_image->h < radius) radius = test_image->h;
		radius /= 2;
		for(n = 0; n < numcorners; n++){
			readpoints[n].x = (int)(x  + radius * sin(2 * pi * (float)n / (float)numcorners));
			readpoints[n].y = (int)(y  + radius * cos(2 * pi * (float)n / (float)numcorners));
			readpoints[n].z = 0;
		}

		gfx_draw_3Dtexture(screen_image, polypoints, readpoints, numcorners, test_image, depth_plane);

		SDL_UpdateRect(screen_image, 0, 0, 0, 0);
		while(GFX_mousebutton_state != 4){
			if(GFX_mousebutton_state == 1 || GFX_mousebutton_state == 2){
				memset((void *)depth_plane, 1000, x_resolution * y_resolution * sizeof(int));
				mx = GFX_mouse_x;
				my = GFX_mouse_y;
				if(mx >= middlex){
					if(GFX_mousebutton_state == 1){
						if(last_button_state != 1){
							closest = 0;
							bestdistance = gfx_hypotenuse(mx, my, polypoints[0].x, polypoints[0].y);
							for(n = 1; n < numcorners; n++){
								distance = gfx_hypotenuse(mx, my, polypoints[n].x, polypoints[n].y);
								if(distance < bestdistance){
									bestdistance = distance;
									closest = n;
								}
							}
							current_area = 1;
						}
						if(current_area == 1){
							polypoints[closest].x = mx;
							polypoints[closest].y = my;
						}
						for(n = 0; n < numcorners; n++){
							drawpoints[n].x = polypoints[n].x;
							drawpoints[n].y = polypoints[n].y;
							drawpoints[n].z = polypoints[n].z;
						}
					}else{
						ang += .1;
						if(ang > 2 * pi) ang -= 2 * pi;
						sine = sin(ang);
						cosine = cos(ang);
						cx = middlex + ((x_resolution - middlex) >> 1);
						cy = y_resolution >> 1;
						cz = 0;
						for(n = 0; n < numcorners; n++){
							drawpoints[n].x = (int)(cosine * (float)(polypoints[n].x - cx) - sine * (float)(polypoints[n].z - cz));
							//drawpoints[n].y = (int)(sine * (float)(polypoints[n].x - cx) + cosine * (float)(polypoints[n].y - cy));
							drawpoints[n].z = (int)(sine * (float)(polypoints[n].x - cx) + cosine * (float)(polypoints[n].z - cz));
							drawpoints[n].x += cx;
							//drawpoints[n].y += cy;
							drawpoints[n].z += cz;
						}
					}
				}else{
					if(last_button_state != 1){
						closest = 0;
						bestdistance = gfx_hypotenuse(mx, my, readpoints[0].x, readpoints[0].y);
						for(n = 1; n < numcorners; n++){
							distance = gfx_hypotenuse(mx, my, readpoints[n].x, readpoints[n].y);
							if(distance < bestdistance){
								bestdistance = distance;
								closest = n;
							}
						}
						current_area = 2;
					}
					if(current_area == 2){
						if(mx >= test_image->w) mx = test_image->w - 1;
						if(my >= test_image->h) my = test_image->h - 1;
						readpoints[closest].x = mx;
						readpoints[closest].y = my;
						gfx_blit(screen_image, 0, 0, test_image);
						for(n = 0; n < numcorners; n++){
							gfx_line(screen_image, readpoints[n].x,
									readpoints[n].y, 
									readpoints[(n + 1) % numcorners].x,
									readpoints[(n + 1) % numcorners].y,
									127, 0, 0);
						}
					}
				}
				last_button_state = GFX_mousebutton_state;
				gfx_solid_box(screen_image, middlex, 0, screen_image->w - 1, screen_image->h - 1, 0, 0, 0);
				gfx_draw_3Dtexture(screen_image, drawpoints, readpoints, numcorners, test_image, depth_plane);
				SDL_UpdateRect(screen_image, 0, 0, 0, 0);
			}else{
				if(last_button_state == 2){
					for(n = 0; n < numcorners; n++){
						polypoints[n].x = drawpoints[n].x;
						polypoints[n].y = drawpoints[n].y;
						polypoints[n].z = drawpoints[n].z;
						ang = 0;
					}
				}
				last_button_state = GFX_mousebutton_state;
			}
			SDL_Delay(5);
			//gfx_copy_dline_to_hline(screen_image, 0, 100, 0, screen_image, 100, 100, GFX_mouse_x, GFX_mouse_y);
			//SDL_UpdateRect(screen_image, 0, 0, 0, 0);
		}
		while(GFX_mousebutton_state == 4);
	}else{
		fprintf(stderr, "errors.\n"); fflush(stderr);
	}

	gfx_close_threads();
	return returnval;
}
